<?php

class Chunk {

    public $UPLOAD_PATH;

    private $_lastChunk = null;
    private $_chunkFolder = '';
    private $_lockFile = '';
    private $_isLast = false;

    public function __construct() {
        ProviderLog::start('Chunk save');
        $this->UPLOAD_PATH = FilesHelper::normalizePath(_PS_UPLOAD_DIR_) . '/chunks/';
        $this->_chunkFolder = $this->UPLOAD_PATH . 'default';
    }

    public function save($info) {
        $validationResult = $this->validate($info);
        if ('' !== $validationResult)
            trigger_error($validationResult, E_USER_ERROR);

        $this->_lastChunk = $info;
        $this->_chunkFolder = $this->UPLOAD_PATH . $info['id'];
        $this->_lockFile = $this->_chunkFolder . '/lock';

        FilesHelper::createDir($this->_chunkFolder);

        $f = fopen($this->_lockFile, 'c');

        if (flock($f, LOCK_EX)) {
            $chunks = array_diff(scandir($this->_chunkFolder), array('.', '..', 'lock'));

            if ((int) $this->_lastChunk['total'] === count($chunks) + 1) {
                $this->_isLast = true;
            }

            if (!empty($this->_lastChunk['blob'])) {
                if (empty($_FILES['content']['tmp_name'])) {
                    ProviderLog::end('Chunk save');
                    return false;
                }
                $content = file_get_contents($_FILES['content']['tmp_name']);
                unlink($_FILES['content']['tmp_name']);
            } else {
                $content = $info['content'];
            }
            FilesHelper::writeFile($this->_chunkFolder . '/' . (int) $info['current'], $content);

            flock($f, LOCK_UN);
            ProviderLog::end('Chunk save');
            return true;
        } else {
            trigger_error('Couldn\'t lock the file: ' . $this->_lockFile, E_USER_NOTICE);
        }
    }

    public function last() {
        return $this->_isLast;
    }

    public function complete() {
        ProviderLog::start('Chunk complete');
        $content = '';
        for ($i = 1, $count = (int) $this->_lastChunk['total']; $i <= $count; $i++) {
            if (!file_exists($this->_chunkFolder . "/$i")) {
                trigger_error('Missing chunk #' . $i . ' : ' . implode(' / ', scandir($this->_chunkFolder)), E_USER_NOTICE);
            }
            $data = FilesHelper::readFile($this->_chunkFolder . "/$i");

            if (!empty($this->_lastChunk['encode']) || !empty($this->_lastChunk['zip'])) {
                $data = base64_decode($data);
            }
            $content .= $data;
        }
        FilesHelper::removeDir($this->_chunkFolder);

        if (!empty($this->_lastChunk['zip'])) {
            $result = $this->unZip($content);
        } else if (!empty($this->_lastChunk['encode'])) {
            $result = array(
                'status' => 'done',
                'data' => rawurldecode($content)
            );
        } else {
            $result = array(
                'status' => 'done',
                'data' => $content
            );
        }

        ProviderLog::end('Chunk complete');
        return $result;
    }

    /**
     * @param $content
     * @return array
     */
    public function unZip($content)
    {
        $tmp = $this->UPLOAD_PATH . 'tmp';
        $archiveFile = $tmp . '/content.zip';
        $unzip = $tmp . '/unzip';
        FilesHelper::writeFile($archiveFile, $content);
        FilesHelper::createDir($unzip);

        $archive = new PclZip($archiveFile);

        $result = array('status' => 'done');
        if (0 == $archive->extract(PCLZIP_OPT_PATH, $unzip)) {
            return array(
                'status' => 'error',
                'message' => "Invalid zip</b></p>Extract error : " . $archive->errorInfo(true)
            );
        }

        if ($result['status'] === 'done' && file_exists($tmp . '/unzip/data')) {
            $result['data'] = FilesHelper::readFile($unzip . '/data');
        } else {
            $result['message'] = 'unzip error';
        }
        FilesHelper::removeDir($tmp);
        return $result;
    }

    private function validate($info) {
        $errors = array();
        if (is_null($info))
            $errors[] = sprintf('Invalid info, json_last_error=%s', json_last_error());
        if (!isset($info['id']) || !$info['id'])
            $errors[] = 'Invalid id';
        if (!isset($info['total']) || (int) $info['total'] < 1)
            $errors[] = 'Invalid chunks total';
        if (!isset($info['current']) || (int) $info['current'] < 1)
            $errors[] = 'Invalid current chunk number';
        if (empty($_FILES['content']) && empty($info['content']))
            $errors[] = 'Invalid chunk content';
        if (count($errors) < 1)
            return '';
        return implode(', ', $errors);
    }
}